public class Directory {
  
  public static void main(String[] args) {
    System.out.println("Welcome to the Muppet Company Directory!\n");
    System.out.println("Reading company directory from disk...\n");
    
    String filename = args[0];
    BST<String,Person> t = PeopleIO.readFromFile(filename);
    
    while(true) {
      // read an option from the user and
      // run selected method
      switch(getOption()) {
        case 1:
          searchEmployee(t);
          break;
        case 2:
          t = addEmployee(t);
          PeopleIO.writeToFile(t, filename);
          break;
        case 3:
          printEntireDirectory(t);
          break;
        default:
          System.exit(0);
      }
    }
  }
  
  private static void searchEmployee(BST<String,Person> t) {
    String lname = readLastName();
    BST<String,Person> pnode = BST.find(t, lname);
    if (pnode == null) {
      System.out.println("\nRESULT: '" + lname + "' not found in directory.\n");
    } else {
      System.out.println(pnode.getValue() + "\n");
    }
  }
  
  private static BST<String,Person> addEmployee(BST<String,Person> t) {
    Person p = readPerson();
    t = BST.insert(t, p.getLastName(), p);
    System.out.println("\nRESULT: '" + p.getFirstName() + " " + p.getLastName() + "' added to directory.\n");
    return t;
  }
  
  private static void printEntireDirectory(BST<String,Person> t) {
    System.out.println("\nRESULT: Entire company directory\n");
    BST.inOrderPrint(t);
    System.out.println();
  }
  
  private static int getOption() {
    int opt = -1;
    while (opt == -1) {
      welcomeMessage();
      opt = readOption();
    }
    return opt;
  }
  
  private static int readOption() {
    String opt = System.console().readLine();
    switch (opt) {
      case "1": return 1;
      case "2": return 2;
      case "3": return 3;
      case "4": return 4;
      default: return -1;
    }
  }
  
  private static String readLastName() {
    System.out.print("\nEnter a last name: ");
    return System.console().readLine();
  }
  
  private static Person readPerson() {
    System.out.print("\nEnter a first name: ");
    String fname = System.console().readLine();
    System.out.print("\nEnter a last name: ");
    String lname = System.console().readLine();
    System.out.print("\nEnter a telephone number: ");
    String tel = System.console().readLine();
    System.out.print("\nEnter an email address: ");
    String email = System.console().readLine();
    return new Person(fname, lname, tel, email);
  }
  
  private static void welcomeMessage() {
    System.out.println("Please choose an option:");
    System.out.println("1. Search for an employee by last name.");
    System.out.println("2. Add an employee to the directory.");
    System.out.println("3. Print entire employee directory.");
    System.out.println("4. Quit this program.");
    System.out.print("\nEnter a number: ");
  }
}