import javax.xml.parsers.*;
import org.xml.sax.SAXException;
import org.w3c.dom.*;
import java.io.*;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*; 
import javax.xml.transform.stream.*;

public class PeopleIO {
  public static BST<String,Person> readFromFile(String filename) {
    // read XML file
    File xmlFile = new File(filename);
    
    // parse XML file
    Document document = null;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      document = builder.parse(xmlFile);
    } catch (Exception e) {
      return null;
    }
    
    // initialize BST
    BST<String,Person> t = null;
    
    // traverse XML and convert to BST
    return parsePeople(document.getDocumentElement(), t);
  }
  
  public static void writeToFile(BST<String,Person> t, String filename) {
    ArrayList<Person> l = BST.inOrderTraverse(t, new ArrayList<>());
    
    Document document = null;
    try {
      DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
      DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
      document = documentBuilder.newDocument();
    } catch (Exception e) {
      System.out.println("Something went wrong.");
    }
    
    Element root = document.createElement("people");
    document.appendChild(root);
    
    // add all people to root
    for (Person p : l) {
      Element person = document.createElement("person");
      root.appendChild(person);

      // first name
      Element fname = document.createElement("fname");
      fname.appendChild(document.createTextNode(p.getFirstName()));
      person.appendChild(fname);
      
      // last name
      Element lname = document.createElement("lname");
      lname.appendChild(document.createTextNode(p.getLastName()));
      person.appendChild(lname);
      
      // tel
      Element tel = document.createElement("tel");
      tel.appendChild(document.createTextNode(p.getTelNum()));
      person.appendChild(tel);
      
      // email
      Element email = document.createElement("email");
      email.appendChild(document.createTextNode(p.getEmail()));
      person.appendChild(email);
    }
    
      try {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
 
        DOMSource source = new DOMSource(document);
        StreamResult result = new StreamResult(filename);
        transformer.transform(source, result);
      } catch (Exception e) {
        // recovery action!
      }
  }
  
  private static BST<String,Person> parsePeople(Node root, BST<String,Person> t) {
    // read in people
    if (root.getNodeName() == "people" && root.hasChildNodes()) {
      NodeList pList = root.getChildNodes();
      
      // run this for each person
      for (int i = 0; i < pList.getLength(); i++) {
        Node n = pList.item(i);
        if(n.getNodeType() == Node.ELEMENT_NODE)
        {	
            Person p = parsePerson(n);
            t = BST.insert(t, p.getLastName(), p);
        }
      }
    }
    
    return t;
  }
  
  private static Person parsePerson(Node n) throws XMLParseException {
    if (n.hasChildNodes()) {
      NodeList pList = n.getChildNodes();
      
      String fname = "";
      String lname = "";
      String tel = "";
      String email = "";
      
      // run this for each person's field
      for (int i = 0; i < pList.getLength(); i++) {
        Node field = pList.item(i);
        String name = field.getNodeName();
        String value = field.getTextContent();
        
        switch (name) {
          case "fname":
            fname = value;
            break;
          case "lname":
            lname = value;
            break;
          case "tel":
            tel = value;
            break;
          case "email":
            email = value;
            break;
          default:
            // ignore
            break;
        }
      }
      
      return new Person(fname, lname, tel, email);
    } else {
      throw new XMLParseException("A person must have fields!");
    }
  }
}